/*
 *  Copyright (C) 2008 hiNt <h1nt@bk.ru>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package mpe;

import java.util.Vector;



public class Document {

    public Vector data;
    public boolean inSelection = false;
    public int curX,  curY,  stringOffset,  vPosition;
    public int selAnchorX,  selAnchorY,  selX1,  selY1,  selX2,  selY2;
    public String filename;
    private int maxLines;

    public Document(String fName, boolean createTemplate, int maxL) {
        filename = fName;
        maxLines = maxL;

        data = new Vector();
        if (createTemplate) {
            String[] initialCode = Utils.splitString(Utils.processMacros(Database.template), "\n");
            for (int i = 0; i < initialCode.length; i++) {
                data.addElement(initialCode[i]);
            }
        }

        curX = stringOffset = vPosition = 0;
        curY = createTemplate ? Database.templLine - 1 : 0;
    }

    public void moveCursor(int dir) {
        /* 0 - Left
         * 1 - Up
         * 2 - Right
         * 3 - Down
         */
        switch (dir) {
            case 0:
                if (curX > 0) {
                    curX--;
                    break;
                } else if (globalY() > 0) {
                    curX = data.elementAt(globalY() - 1).toString().length();
                }
            // Выделение
            case 1:
                if (globalY() > 0) {
                    curY--;
                    if (curY < 0 && vPosition > 0) {
                        curY = 0;
                        vPosition--;
                    }
                    if (curX > data.elementAt(globalY()).toString().length()) {
                        curX = data.elementAt(globalY()).toString().length();
                    }
                }
                break;
            case 2:
                if (curX < data.elementAt(globalY()).toString().length()) {
                    curX++;
                    break;
                } else if (data.size() > globalY() + 1 && globalY() < data.size()) {
                    curX = 0;
                }
                

            case 3:
                if (globalY() < data.size() - 1) {
                    curY++;
                    if (curY == maxLines) {
                        curY = maxLines - 1;
                        vPosition++;
                    }
                    if (curX > data.elementAt(globalY()).toString().length()) {
                        curX = data.elementAt(globalY()).toString().length();
                    }
                }
                break;
        }
    }

    public void Insert(String str) {
        String[] s = Utils.splitString(str, "\n");
        if (inSelection) {
            Backspace();
        }
        if (curX == 0) {
            if (s.length > 1) {
                String temp = data.elementAt(globalY()).toString();
                data.setElementAt(s[0], globalY());
                for (int i = 1; i < s.length - 1; i++) {
                    curY++;
                    data.insertElementAt(s[i], globalY());
                }
                curY++;
                data.insertElementAt(s[s.length - 1] + temp, globalY());
            } else {
                data.setElementAt(s[0] + data.elementAt(globalY()).toString(), globalY());
            }
            setCursor(s[s.length - 1].length(), globalY());
        } else if (curX == data.elementAt(globalY()).toString().length()) {
            data.setElementAt(data.elementAt(globalY()).toString() + s[0], globalY());
            if (s.length > 1) {
                for (int i = 1; i < s.length; i++) {
                    curY++;
                    data.insertElementAt(s[i], globalY());
                }
            }
            setCursor(data.elementAt(globalY()).toString().length(), globalY());
        } else {
            String temp = data.elementAt(globalY()).toString();
            if (s.length > 1) {
                data.setElementAt(temp.substring(0, curX) + s[0], globalY());
                for (int i = 1; i < s.length - 1; i++) {
                    curY++;
                    data.insertElementAt(s[i], globalY());
                }
                curY++;
                data.insertElementAt(s[s.length - 1] + temp.substring(curX), globalY());
                setCursor(s[s.length - 1].length(), globalY());
            } else {
                data.setElementAt(temp.substring(0, curX) + s[0] + temp.substring(curX), globalY());
                setCursor(temp.substring(0, curX).length() + s[0].length(), globalY());
            }
        }
    }

    public void Backspace() {
        if (inSelection) {
            calcSelection();
            setCursor(selX1, selY1);
            for (int i = selY1; i <= selY2; i++) {
                if (selY1 == selY2) {
                    data.setElementAt(data.elementAt(selY1).toString().substring(0, selX1) + data.elementAt(selY1).toString().substring(selX2), selY1);
                } else if (i == selY1) {
                    if (selX1 == 0) {
                        data.removeElementAt(i);
                        selY2--;
                        i--;
                    } else {
                        data.setElementAt(data.elementAt(i).toString().substring(0, selX1), i);
                    }
                } else if (i == selY2) {
                    if (selX2 == data.elementAt(i).toString().length()) {
                        data.removeElementAt(i);
                        selY2--;
                        i--;
                    } else {

                        data.setElementAt(data.elementAt(i).toString().substring(selX2), i);
                    }
                } else {
                    data.removeElementAt(i);
                    selY2--;
                    i--;
                }
            }
            endSelection();
            System.out.println("ɾ");
            
        } else if (curX != 0 || globalY() != 0) {
            if (curX > 0) {
                data.setElementAt(Utils.stringDelete(data.elementAt(globalY()).toString(), --curX), globalY());
            } else {
                curY--;
                curX = data.elementAt(globalY()).toString().length();
                data.setElementAt(data.elementAt(globalY()).toString() + data.elementAt(globalY() + 1).toString(), globalY());
                data.removeElementAt(globalY() + 1);
                if (curY < 0 && vPosition > 0) {
                    curY = 0;
                    vPosition--;
                }
            }
            System.out.println("ɾ");
        }
    }

    public String Copy() {
        String buffer = "";
        if (inSelection) {
            calcSelection();
            if (selY1 == selY2) {
                buffer = data.elementAt(selY1).toString().substring(selX1, selX2);
            } else {
                for (int i = selY1; i <= selY2; i++) {
                    if (i == selY1) {
                        buffer = data.elementAt(i).toString().substring(selX1) + "\n";
                    } else if (i == selY2) {
                        buffer += data.elementAt(i).toString().substring(0, selX2);
                    } else {
                        buffer += data.elementAt(i).toString() + "\n";
                    }
                }
            }
        }

        return buffer;
    }
    public String GetSelectedText()
    {
        String buffer = "";
        if (inSelection) {
            calcSelection();
            if (selY1 == selY2) {
                buffer = data.elementAt(selY1).toString().substring(selX1, selX2);
            } else {
                for (int i = selY1; i <= selY2; i++) {
                    if (i == selY1) {
                        buffer = data.elementAt(i).toString().substring(selX1) + "\n";
                    } else if (i == selY2) {
                        buffer += data.elementAt(i).toString().substring(0, selX2);
                    } else {
                        buffer += data.elementAt(i).toString() + "\n";
                    }
                }
            }
        }

        return buffer;
    }
    public String Cut() {
        String buffer = "";
        if (inSelection) {
            buffer = Copy();
            Backspace();
        }

        return buffer;
    }

    public void setCursor(int x, int y) {
        /*if (y >= data.size()) {
            y = data.size() - 1;
        }
        if (x >= data.elementAt(y).toString().length()) {
            x = data.elementAt(y).toString().length() - 1;
        }*/
        curX = x;
        if (y < vPosition) {
            curY = 0;
            vPosition = y;
        } else if (y > vPosition + maxLines - 1) {
            curY = maxLines - 1;
            vPosition = y - maxLines + 1;
        } else {
            curY = y - vPosition;
        }
    }

    public int getCursor(int coord) {
        return coord == Constants.CUR_X ? curX : globalY();
    }

    public final int globalY() {
        return curY + vPosition;
    }

    public void startSelection() {
        inSelection = true;
        selAnchorX = curX;
        selAnchorY = globalY();
    }

    public void endSelection() {
        inSelection = false;
    }

    public void calcSelection() {
        if (selAnchorY == globalY()) {
            selX1 = selAnchorX < curX ? selAnchorX : curX;
            selX2 = selAnchorX < curX ? curX : selAnchorX;
            selY1 = selY2 = globalY();
        } else {
            selX1 = selAnchorY < globalY() ? selAnchorX : curX;
            selX2 = selAnchorY < globalY() ? curX : selAnchorX;
            selY1 = selAnchorY < globalY() ? selAnchorY : globalY();
            selY2 = selAnchorY < globalY() ? globalY() : selAnchorY;
        }
    }

    public void setMaxLines(int maxL) {
        maxLines = maxL;
        setCursor(curX, curY);
    }
}
