/*
 *  Copyright (C) 2008 hiNt <h1nt@bk.ru>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package mpe;

import java.io.InputStream;
import java.io.OutputStream;
import java.util.Enumeration;
import javax.microedition.io.Connector;
import javax.microedition.io.file.FileConnection;
import javax.microedition.io.file.FileSystemRegistry;
import javax.microedition.lcdui.Command;
import javax.microedition.lcdui.CommandListener;
import javax.microedition.lcdui.Displayable;
import javax.microedition.lcdui.Image;
import javax.microedition.lcdui.List;
import javax.microedition.lcdui.TextBox;
import javax.microedition.lcdui.TextField;

public class FileManager implements CommandListener {

    private final int INPUT_FILENAME = 0;
    private final int INPUT_DIRNAME = 1;
    private final int INPUT_RENAME = 2;
    private int input;
    private List lFiles;
    private String curPath;
    private boolean isRoot,  opening;
    private Command cmdSaveHere,  cmdOk,  cmdCancel,  cmdDelete,  cmdCreateDir,  cmdRename;
    private TextBox tbInput;
    private MPE mpe;

    public FileManager(MPE mpe) {
        this.mpe = mpe;
        curPath = "";
        isRoot = true;
        cmdSaveHere = new Command(mpe.lr.get("26"), Command.OK, 0);
        cmdOk = new Command(mpe.lr.get("27"), Command.OK, 0);
        cmdCreateDir = new Command(mpe.lr.get("64"), Command.ITEM, 1);
        cmdDelete = new Command(mpe.lr.get("63"), Command.ITEM, 2);
        cmdRename = new Command(mpe.lr.get("74"), Command.ITEM, 3);
        cmdCancel = new Command(mpe.lr.get("65"), Command.BACK, 5);
    }

    public void commandAction(Command c, Displayable d) {
        if (c == List.SELECT_COMMAND) {
            if (lFiles.getSelectedIndex() == 0 && !isRoot) {
                int n = curPath.substring(0, curPath.length() - 1).lastIndexOf('/');
                if (n < 0) {
                    showRoot();
                } else {
                    showDir(curPath = curPath.substring(0, n) + '/');
                }
            } else if (lFiles.getString(lFiles.getSelectedIndex()).charAt(lFiles.getString(lFiles.getSelectedIndex()).length() - 1) == '/') {
                showDir(curPath += lFiles.getString(lFiles.getSelectedIndex()));
            } else {
                if (opening) {
                    open(curPath + lFiles.getString(lFiles.getSelectedIndex()));
                } else {
                    save(curPath + lFiles.getString(lFiles.getSelectedIndex()));
                }
            }
        } else if (c == cmdSaveHere) {
            input = INPUT_FILENAME;
            tbInput = new TextBox(mpe.lr.get("33"), "", 256, TextField.ANY);
            tbInput.addCommand(cmdOk);
            tbInput.addCommand(cmdCancel);
            tbInput.setCommandListener(this);
            mpe.display.setCurrent(tbInput);
        } else if (c == cmdCreateDir) {
            input = INPUT_DIRNAME;
            tbInput = new TextBox(mpe.lr.get("73"), "", 256, TextField.ANY);
            tbInput.addCommand(cmdOk);
            tbInput.addCommand(cmdCancel);
            tbInput.setCommandListener(this);
            mpe.display.setCurrent(tbInput);
        } else if (c == cmdRename && lFiles.getSelectedIndex() > 0) {
            input = INPUT_RENAME;
            tbInput = new TextBox(mpe.lr.get("75"),
                    lFiles.getString(lFiles.getSelectedIndex()).charAt(lFiles.getString(lFiles.getSelectedIndex()).length() - 1) == '/' ? lFiles.getString(lFiles.getSelectedIndex()).substring(0, lFiles.getString(lFiles.getSelectedIndex()).length() - 1) : lFiles.getString(lFiles.getSelectedIndex()), 256, TextField.ANY);
            tbInput.addCommand(cmdOk);
            tbInput.addCommand(cmdCancel);
            tbInput.setCommandListener(this);
            mpe.display.setCurrent(tbInput);
        } else if (c == cmdDelete && lFiles.getSelectedIndex() > 0) {
            delete(curPath + lFiles.getString(lFiles.getSelectedIndex()));
        } else if (c == cmdOk) {
            if (d == tbInput) {
                switch (input) {
                    case INPUT_FILENAME:
                        save(curPath + tbInput.getString());
                        break;
                    case INPUT_DIRNAME:
                        createDir(curPath + tbInput.getString());
                        break;
                    case INPUT_RENAME:
                        rename(curPath + lFiles.getString(lFiles.getSelectedIndex()), tbInput.getString());
                        break;
                }
            } else {
                mpe.display.setCurrent(lFiles);
            }
        } else if (c == cmdCancel) {
            if (d == lFiles) {
                mpe.display.setCurrent(mpe.w);
                System.gc();
            } else if (d == tbInput) {
                mpe.display.setCurrent(lFiles);
            }
        }
    }

    public void show(boolean opening) {
        this.opening = opening;
        if (curPath.equals("") || isRoot) {
            showRoot();
        } else {
            showDir(curPath);
        }
    }

    private void showRoot() {
        final FileManager fm = this;
        new Thread(new Runnable() {

            public void run() {
                lFiles = new List(opening ? mpe.lr.get("34") : mpe.lr.get("35"), List.IMPLICIT);
                Enumeration e = FileSystemRegistry.listRoots();
                try {
                    while (e.hasMoreElements()) {
                        lFiles.append(e.nextElement().toString(), Image.createImage("/res/disk.png"));
                    }
                } catch (Exception ex) {
                }
                isRoot = true;
                curPath = "";
                lFiles.addCommand(cmdCancel);
                lFiles.setCommandListener(fm);
                mpe.display.setCurrent(lFiles);
            }
        }).start();
    }

    private void showDir(final String path) {
        final FileManager fm = this;
        new Thread(new Runnable() {

            public void run() {
                lFiles = new List(path, List.IMPLICIT);
                try {
                    FileConnection fconn = (FileConnection) Connector.open("file:///" + path);
                    Enumeration e = fconn.list("*", true);
                    lFiles.append("..", Image.createImage("/res/folder.png"));
                    int c = 1;
                    while (e.hasMoreElements()) {
                        String temp = e.nextElement().toString();
                        if (temp.charAt(temp.length() - 1) == '/') {
                            lFiles.insert(c++, temp, Image.createImage("/res/folder.png"));
                        } else {
                            lFiles.append(temp, Image.createImage("/res/file.png"));
                        }
                    }
                    fconn.close();
                } catch (Exception e) {
                }
                isRoot = false;
                if (!opening) {
                    lFiles.addCommand(cmdSaveHere);
                }
                lFiles.addCommand(cmdCreateDir);
                lFiles.addCommand(cmdDelete);
                lFiles.addCommand(cmdRename);
                lFiles.addCommand(cmdCancel);
                lFiles.setCommandListener(fm);
                mpe.display.setCurrent(lFiles);
            }
        }).start();
    }

    private void rename(final String from, final String to) {
        new Thread(new Runnable() {

            public void run() {
                try {
                    FileConnection fconn = (FileConnection) Connector.open("file:///" + from);
                    fconn.rename(to);
                    fconn.close();
                } catch (Exception e) {
                } finally {
                    showDir(curPath);
                }
            }
        }).start();
    }

    private void delete(final String fn) {
        new Thread(new Runnable() {

            public void run() {
                try {
                    FileConnection fconn = (FileConnection) Connector.open("file:///" + fn);
                    fconn.delete();
                    fconn.close();
                } catch (Exception e) {
                } finally {
                    showDir(curPath);
                }
            }
        }).start();
    }

    private void createDir(final String dirName) {
        new Thread(new Runnable() {

            public void run() {
                try {
                    FileConnection fconn = (FileConnection) Connector.open("file:///" + dirName);
                    fconn.mkdir();
                    fconn.close();
                } catch (Exception e) {
                } finally {
                    showDir(curPath);
                }
            }
        }).start();
    }

    public void open(final String fn) {
        new Thread(new Runnable() {

            public void run() {
                if (!mpe.w.openDocument(fn)) {
                    try {
                        FileConnection fconn = (FileConnection) Connector.open("file:///" + fn);
                        InputStream is = fconn.openInputStream();
                        long l = fconn.fileSize();
                        if (l < Integer.MAX_VALUE) {
                            byte[] b = new byte[(int) fconn.fileSize()];
                            is.read(b);
                            String str=Utils.byteArrayToString(b, "Windows-1251");
                            str=str.replace('\t',' ');
                            String[] s = Utils.splitString(str, Database.separator == 0 ? "\n" : (Database.separator == 1 ? "\r\n" : "\r"));
                            b = null;
                            Document doc = new Document(fn, false, mpe.w.maxLines);
                            for (int i = 0; i < s.length; i++) {
                                doc.data.addElement(s[i]);
                            }
                            mpe.w.addDocument(doc);
                        }
                        is.close();
                        fconn.close();
                    } catch (Exception e) {
                    }
                }
                mpe.w.repaint();
                mpe.display.setCurrent(mpe.w);
                System.gc();
            }
        }).start();
    }

    public void save(final String fn) {
        new Thread(new Runnable() {

            public void run() {
                try {
                    FileConnection fconn = (FileConnection) Connector.open("file:///" + fn);
                    if (fconn.exists()) {
                        fconn.truncate(0);
                    } else {
                        fconn.create();
                    }
                    OutputStream os = fconn.openOutputStream();
                    String temp = "";
                    for (int i = 0; i < mpe.w.getDocument().data.size(); i++) {
                        temp += mpe.w.getDocument().data.elementAt(i).toString() + (i != mpe.w.getDocument().data.size() - 1 ? "\n" : "");
                    }
                    os.write(Utils.encodeUTF8(temp));
                    temp = null;
                    mpe.w.getDocument().filename = fn;
                    os.close();
                    fconn.close();
                } catch (Exception ex) {
                }
                mpe.display.setCurrent(mpe.w);
                System.gc();
            }
        }).start();
    }

    public void restore() {
        String[] files = Utils.splitString(Database.lastsession, "\n");
        for (int i = 0; i < files.length; i++) {
            try {
                FileConnection fconn = (FileConnection) Connector.open("file:///" + files[i]);
                InputStream is = fconn.openInputStream();
                long l = fconn.fileSize();
                if (l < Integer.MAX_VALUE) {
                    byte[] b = new byte[(int) l];
                    is.read(b);
                    String[] s = Utils.splitString(Utils.byteArrayToString(b, "Windows-1251"), Database.separator == 0 ? "\n" : (Database.separator == 1 ? "\r\n" : "\r"));
                    b = null;
                    Document doc = new Document(files[i], false, mpe.w.maxLines);
                    for (int j = 0; j < s.length; j++) {
                        doc.data.addElement(s[j]);
                    }
                    mpe.w.addDocument(doc);
                }
                is.close();
                fconn.close();
            } catch (Exception e) {
            }
        }
        mpe.w.repaint();
        mpe.display.setCurrent(mpe.w);
        System.gc();
    }
}
