package org.pabloid.util;

public class Random
{

    private long seed;
    private final static long multiplier = 0x5DEECE66DL;
    private final static long addend = 0xBL;
    private final static long mask = (1L << 48) - 1;

    public Random()
    {
        this(++seedUniquifier + System.currentTimeMillis() ^ multiplier);
    }
    private static volatile long seedUniquifier = 8682522807148012L;

    public Random(long seed)
    {
        this.seed = 0;
        setSeed(seed);
    }

    synchronized public void setSeed(long seed)
    {
        seed = (seed ^ multiplier) & mask;
        this.seed = seed;
    }

    protected int next(int bits)
    {
        long oldseed, nextseed;
        oldseed = this.seed;
        do
            nextseed = (oldseed * multiplier + addend) & mask;
        while (!compareAndSet(oldseed, nextseed));
        return (int) (nextseed >>> (48 - bits));
    }

    private boolean compareAndSet(long oldseed, long newseed)
    {
        long s = (oldseed * multiplier + addend) ^ newseed;
        if (s != seed)
            s = seed;
        else
            return false;
        return true;
    }

    public void nextBytes(byte[] bytes)
    {
        for (int i = 0, len = bytes.length; i < len;)
            for (int rnd = nextInt(),
                    n = Math.min(len - i, 4);
                    n-- > 0; rnd >>= 1)
                bytes[i++] = (byte) rnd;
    }

    public byte[] nextBytes(int length)
    {
        byte[] data = new byte[length];
        nextBytes(data);
        return data;
    }

    public String nextString(int length)
    {
        byte data[] = new byte[length];
        nextBytes(data);
        return new String(data);
    }

    public String nextBase64String(int length)
    {
        byte[] data = nextBytes(length);
        return Base64.encode(data);
    }

    public int nextInt()
    {
        return next(32);
    }

    public byte nextByte()
    {
        return (byte) next(8);
    }

    public short nextShort()
    {
        return (short) next(16);
    }

    public char nextChar()
    {
        return (char) Math.abs(next(16));
    }

    public int nextInt(int n)
    {
        if (n <= 0)
            throw new IllegalArgumentException("Random.nextInt(int n): n must be positive");

        if ((n & -n) == n)  // i.e., n is a power of 2
            return (int) ((n * (long) next(31)) >> 31);

        int bits, val;
        do
        {
            bits = next(31);
            val = bits % n;
        } while (bits - val + (n - 1) < 0);
        return val;
    }

    public long nextLong()
    {
        return ((long) (next(32)) << 32) + next(32);
    }

    public boolean nextBoolean()
    {
        return next(1) != 0;
    }

    public float nextFloat()
    {
        return next(24) / ((float) (1 << 24));
    }

    public double nextDouble()
    {
        return (((long) (next(26)) << 27) + next(27)) / (double) (1L << 53);
    }
}
